using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using System.Collections.Generic;
using OSGeo.FDO.Commands.Schema;
using OSGeo.FDO.Schema;
using OSGeo.FDO.Commands.Feature;
using OSGeo.FDO;
using OSGeo.FDO.Connections;
using OSGeo.FDO.ClientServices;

public partial class Default2 : System.Web.UI.Page
{
    IConnection connection;

    protected void Page_Load(object sender, EventArgs e)
    {

        Open(Server.MapPath("~/App_LocalResources"));
        Read();

        // get the schema description
        IDescribeSchema descSchema = (IDescribeSchema)connection.CreateCommand(OSGeo.FDO.Commands.CommandType.CommandType_DescribeSchema);
        FeatureSchemaCollection schemas = descSchema.Execute();
        FeatureSchema schema = schemas[0];

        // copy class def
        ClassCollection classes = schema.Classes;

        // add the dest class
        FeatureClass featdest = new FeatureClass(System.IO.Path.GetRandomFileName().Replace(".",""), "");
        featdest.IsAbstract = false;

        DataPropertyDefinition featid = new DataPropertyDefinition("FeatId", "integer");
        featid.DataType = DataType.DataType_Int32;
        featid.IsAutoGenerated = true;
        featid.Nullable = false;

        DataPropertyDefinition id = new DataPropertyDefinition("Id", "decimal");
        id.DataType = DataType.DataType_Decimal;
        id.Precision = 10;
        id.Scale = 10;

        DataPropertyDefinition street = new DataPropertyDefinition("Street", "text");
        street.DataType = DataType.DataType_String;
        street.Length = 64;

        DataPropertyDefinition area = new DataPropertyDefinition("Area", "double");
        area.DataType = DataType.DataType_Decimal;
        area.Precision = 20;
        area.Scale = 20;

        DataPropertyDefinition vacant = new DataPropertyDefinition("Vacant", "boolean");
        vacant.DataType = DataType.DataType_Boolean;

        DataPropertyDefinition birthday = new DataPropertyDefinition("Birthday", "date");
        birthday.DataType = DataType.DataType_DateTime;

        //// assemble the feature class
        PropertyDefinitionCollection properties = featdest.Properties;
        properties.Add(featid);
        properties.Add(id);
        properties.Add(street);
        properties.Add(area);
        properties.Add(vacant);
        properties.Add(birthday);
        DataPropertyDefinitionCollection identities = featdest.IdentityProperties;
        identities.Add(featid);

        // add class
        classes.Add(featdest);

        // apply schema            
        IApplySchema apply = (IApplySchema)connection.CreateCommand(OSGeo.FDO.Commands.CommandType.CommandType_ApplySchema);
        apply.FeatureSchema = schema;
        apply.Execute();

        Read();
        Close();
        Response.End();
    }

    public void Open(string defaultDirectory)
    {
        IConnectionManager connectionManager = FeatureAccessManager.GetConnectionManager();
        connection = connectionManager.CreateConnection("OSGeo.SHP.3.2");

        IConnectionPropertyDictionary connectionPropertyDictionary;
        connectionPropertyDictionary = connection.ConnectionInfo.ConnectionProperties;

        connectionPropertyDictionary.SetProperty("DefaultFileLocation", defaultDirectory);
        connectionPropertyDictionary.SetProperty("TemporaryFileLocation", defaultDirectory);
        
        connection.Open();        
    }

    public void Read()
    {
        string className = "testfeature";

        // get class def
        IDescribeSchema descSchema = (IDescribeSchema)connection.CreateCommand(OSGeo.FDO.Commands.CommandType.CommandType_DescribeSchema);
        FeatureSchemaCollection schemas = descSchema.Execute();
        ClassDefinition classDef = schemas[0].Classes[schemas[0].Classes.IndexOf(className)];

        // read first
        ISelect sel = (ISelect)connection.CreateCommand(OSGeo.FDO.Commands.CommandType.CommandType_Select);
        sel.SetFeatureClassName(className);
        sel.SetFilter("FeatId=1");
        IReader reader = sel.Execute();
        while (reader.ReadNext())
        {
            foreach (PropertyDefinition pdef in classDef.Properties)
            {
                switch (pdef.PropertyType)
                {
                    case PropertyType.PropertyType_DataProperty:
                        if (!reader.IsNull(pdef.Name))
                        {
                            DataPropertyDefinition ddef = (DataPropertyDefinition)pdef;
                            switch (ddef.DataType)
                            {
                                case DataType.DataType_Int16: Response.Write(pdef.Name +"="+ reader.GetInt16(pdef.Name) + "<br>"); break;
                                case DataType.DataType_Int32: Response.Write(pdef.Name +"="+ reader.GetInt32(pdef.Name) + "<br>"); break;
                                case DataType.DataType_Int64: Response.Write(pdef.Name +"="+ reader.GetInt64(pdef.Name) + "<br>"); break;
                                case DataType.DataType_Single: Response.Write(pdef.Name +"="+ reader.GetSingle(pdef.Name) + "<br>"); break;
                                case DataType.DataType_Decimal:
                                case DataType.DataType_Double: Response.Write(pdef.Name +"="+ reader.GetDouble(pdef.Name) + "<br>"); break;
                                default:
                                    Response.Write(pdef.Name +"="+ reader.GetString(pdef.Name) + "<br>"); break;
                            }
                        }
                        break;
                }                
            }
            Response.Write("<hr>");
        }
        Response.Write("<b>DONE</b><br><br>");
        reader.Close();
    }

    public void Close()
    {
        connection.Close();
        connection = null;
    }

}
